@@ -23,13 +23,17 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 
+#include <menu-cache.h>
+
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+
 #include "panel.h"
 #include "misc.h"
 #include "plugin.h"
 #include "bg.h"
 
-#include "ptk-app-menu.h"
-
 #include "dbg.h"
 
 /*
@@ -42,8 +46,7 @@
  */
 
 typedef struct {
-    GtkTooltips *tips;
-    GtkWidget *menu, *box, *bg, *label;
+    GtkWidget *menu, *box, *img, *label;
     char *fname, *caption;
     gulong handler_id;
     int iconsize, paneliconsize;
@@ -52,30 +55,44 @@ typedef struct {
     char* config_data;
     int sysmenu_pos;
     char *config_start, *config_end;
+
+    MenuCache* menu_cache;
+    gpointer reload_notify;
 } menup;
 
 static guint idle_loader = 0;
 
+GQuark SYS_MENU_ITEM_ID = 0;
+
+/* a single-linked list storing all panels */
+extern GSList* all_panels;
+
+
 static void
 menu_destructor(Plugin *p)
 {
     menup *m = (menup *)p->priv;
 
-    ENTER;
-
     if( G_UNLIKELY( idle_loader ) )
     {
         g_source_remove( idle_loader );
         idle_loader = 0;
     }
+
     if( m->has_system_menu )
         p->panel->system_menus = g_slist_remove( p->panel->system_menus, p );
-
-    g_signal_handler_disconnect(G_OBJECT(m->bg), m->handler_id);
+    g_signal_handler_disconnect(G_OBJECT(m->img), m->handler_id);
     gtk_widget_destroy(m->menu);
     /* The widget is destroyed in plugin_stop().
     gtk_widget_destroy(m->box);
     */
+
+    if( m->menu_cache )
+    {
+        menu_cache_remove_reload_notify(m->menu_cache, m->reload_notify);
+        menu_cache_unref( m->menu_cache );
+    }
+
     g_free(m->fname);
     g_free(m->caption);
     g_free(m);
@@ -139,28 +156,396 @@ menu_pos(GtkMenu *menu, gint *x, gint *y, gboolean *push_in, GtkWidget *widget)
     RET();
 }
 
+static void on_menu_item( GtkMenuItem* mi, MenuCacheItem* item )
+{
+    lxpanel_launch_app( menu_cache_app_get_exec(MENU_CACHE_APP(item)),
+            NULL, menu_cache_app_get_use_terminal(MENU_CACHE_APP(item)));
+}
+
+/* load icon when mapping the menu item to speed up */
+static void on_menu_item_map(GtkWidget* mi, MenuCacheItem* item)
+{
+  GtkImage* img = GTK_IMAGE(gtk_image_menu_item_get_image(GTK_IMAGE_MENU_ITEM(mi)));
+    if( img )
+    {
+        if( gtk_image_get_storage_type(img) == GTK_IMAGE_EMPTY )
+        {
+            GdkPixbuf* icon;
+            int w, h;
+            /* FIXME: this is inefficient */
+            gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &w, &h);
+            item = g_object_get_qdata(G_OBJECT(mi), SYS_MENU_ITEM_ID);
+            icon = lxpanel_load_icon(menu_cache_item_get_icon(item), MAX(w,h), TRUE);
+            gtk_image_set_from_pixbuf(img, icon);
+            g_object_unref(icon);
+        }
+    }
+}
+
+static void on_menu_item_style_set(GtkWidget* mi, GtkStyle* prev, MenuCacheItem* item)
+{
+    /* reload icon */
+    on_menu_item_map(mi, item);
+}
+
+static void on_add_menu_item_to_desktop(GtkMenuItem* item, MenuCacheApp* app)
+{
+    char* dest;
+    char* src;
+    g_debug("app: %p", app);
+    const char* desktop = g_get_user_special_dir(G_USER_DIRECTORY_DESKTOP);
+    int dir_len = strlen(desktop);
+    int basename_len = strlen(menu_cache_item_get_id(MENU_CACHE_ITEM(app)));
+    int dest_fd;
+
+    dest = g_malloc( dir_len + basename_len + 6 + 1 + 1 );
+    memcpy(dest, desktop, dir_len);
+    dest[dir_len] = '/';
+    memcpy(dest + dir_len + 1, menu_cache_item_get_id(MENU_CACHE_ITEM(app)), basename_len + 1);
+
+    /* if the destination file already exists, make a unique name. */
+    if( g_file_test( dest, G_FILE_TEST_EXISTS ) )
+    {
+        memcpy( dest + dir_len + 1 + basename_len - 8 /* .desktop */, "XXXXXX.desktop", 15 );
+        dest_fd = g_mkstemp(dest);
+        if( dest_fd >= 0 )
+            chmod(dest, 0600);
+    }
+    else
+    {
+        dest_fd = creat(dest, 0600);
+    }
+
+    if( dest_fd >=0 )
+    {
+        char* data;
+        gsize len;
+        src = menu_cache_item_get_file_path(MENU_CACHE_ITEM(app));
+        if( g_file_get_contents(src, &data, &len, NULL) )
+        {
+            write( dest_fd, data, len );
+            g_free(data);
+        }
+        close(dest_fd);
+        g_free(src);
+    }
+    g_free(dest);
+}
+
+/* TODO: add menu item to panel */
+static void on_add_menu_item_to_panel(GtkMenuItem* item, MenuCacheApp* app)
+{
+    /* Find a penel containing launchbar applet.
+     * The launchbar with most buttons will be choosen if
+     * there are several launchbar applets loaded.
+     */
+    GSList* l;
+    Plugin* lb = NULL;
+    int n_btns = -1;
+
+    for(l = all_panels; !lb && l; l = l->next)
+    {
+        Panel* panel = (Panel*)l->data;
+        GList* pl;
+        for(pl=panel->plugins; pl; pl = pl->next)
+        {
+            Plugin* plugin = (Plugin*)pl;
+            if( strcmp(plugin->class->type, "launchbar") == 0 )
+            {
+                /* FIXME: should we let the users choose which launcherbar to add the btn? */
+                break;
+#if 0
+                int n = launchbar_get_n_btns(plugin);
+                if( n > n_btns )
+                {
+                    lb = plugin;
+                    n_btns = n;
+                }
+#endif
+            }
+        }
+    }
+
+    if( ! lb ) /* launchbar is not currently in use */
+    {
+        /* FIXME: add a launchbar plugin to the panel which has a menu, too. */
+    }
+
+    if( lb )
+    {
+
+    }
+}
+
+static void on_menu_item_properties(GtkMenuItem* item, MenuCacheApp* app)
+{
+    /* FIXME: if the source desktop is in AppDir other then default
+     * applications dirs, where should we store the user-specific file?
+    */
+    char* ifile = menu_cache_item_get_file_path(MENU_CACHE_ITEM(app));
+    char* ofile = g_build_filename(g_get_user_data_dir(), "applications",
+				   menu_cache_item_get_file_basename(MENU_CACHE_ITEM(app)), NULL);
+    char* argv[] = {
+        "lxshortcut",
+        "-i",
+        NULL,
+        "-o",
+        NULL,
+        NULL};
+    argv[2] = ifile;
+    argv[4] = ofile;
+    g_spawn_async( NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, NULL );
+    g_free( ifile );
+    g_free( ofile );
+}
+
+/* This following function restore_grabs is taken from menu.c of
+ * gnome-panel.
+ */
+/*most of this function stolen from the real gtk_menu_popup*/
+static void restore_grabs(GtkWidget *w, gpointer data)
+{
+    GtkWidget *menu_item = data;
+    GtkMenu *menu = GTK_MENU(menu_item->parent);
+    GtkWidget *xgrab_shell;
+    GtkWidget *parent;
+
+    /* Find the last viewable ancestor, and make an X grab on it
+    */
+    parent = GTK_WIDGET (menu);
+    xgrab_shell = NULL;
+    while (parent)
+    {
+        gboolean viewable = TRUE;
+        GtkWidget *tmp = parent;
+
+        while (tmp)
+        {
+            if (!GTK_WIDGET_MAPPED (tmp))
+            {
+                viewable = FALSE;
+                break;
+            }
+            tmp = tmp->parent;
+        }
+
+        if (viewable)
+            xgrab_shell = parent;
+
+        parent = GTK_MENU_SHELL (parent)->parent_menu_shell;
+    }
+
+    /*only grab if this HAD a grab before*/
+    if (xgrab_shell && (GTK_MENU_SHELL (xgrab_shell)->have_xgrab))
+    {
+        if (gdk_pointer_grab (xgrab_shell->window, TRUE,
+                    GDK_BUTTON_PRESS_MASK |
+                    GDK_BUTTON_RELEASE_MASK |
+                    GDK_ENTER_NOTIFY_MASK |
+                    GDK_LEAVE_NOTIFY_MASK,
+                    NULL, NULL, 0) == 0)
+        {
+            if (gdk_keyboard_grab (xgrab_shell->window, TRUE,
+                    GDK_CURRENT_TIME) == 0)
+                GTK_MENU_SHELL (xgrab_shell)->have_xgrab = TRUE;
+            else
+                gdk_pointer_ungrab (GDK_CURRENT_TIME);
+        }
+    }
+    gtk_grab_add (GTK_WIDGET (menu));
+}
+
+static gboolean on_menu_button_press(GtkWidget* mi, GdkEventButton* evt, MenuCacheItem* data)
+{
+    if( evt->button == 3 )  /* right */
+    {
+        char* tmp;
+        GtkWidget* item;
+        GtkMenu* p = GTK_MENU(gtk_menu_new());
+/*
+        item = gtk_menu_item_new_with_label(_("Add to desktop panel"));
+        g_signal_connect(item, "activate", G_CALLBACK(on_add_menu_item_to_panel), data);
+        gtk_menu_shell_append(p, item);
+*/
+        item = gtk_menu_item_new_with_label(_("Add to desktop"));
+        g_signal_connect(item, "activate", G_CALLBACK(on_add_menu_item_to_desktop), data);
+        gtk_menu_shell_append(GTK_MENU_SHELL(p), item);
+
+        tmp = g_find_program_in_path("lxshortcut");
+        if( tmp )
+        {
+            item = gtk_separator_menu_item_new();
+            gtk_menu_shell_append(GTK_MENU_SHELL(p), item);
+
+            item = gtk_menu_item_new_with_label(_("Properties"));
+            g_signal_connect(item, "activate", G_CALLBACK(on_menu_item_properties), data);
+            gtk_menu_shell_append(GTK_MENU_SHELL(p), item);
+            g_free(tmp);
+        }
+        g_signal_connect(p, "selection-done", G_CALLBACK(gtk_widget_destroy), NULL);
+        g_signal_connect(p, "deactivate", G_CALLBACK(restore_grabs), mi);
+
+        gtk_widget_show_all(GTK_WIDGET(p));
+        gtk_menu_popup(p, NULL, NULL, NULL, NULL, 0, evt->time);
+        return TRUE;
+    }
+    return FALSE;
+}
+
+static GtkWidget* create_item( MenuCacheItem* item )
+{
+    GtkWidget* mi;
+    if( menu_cache_item_get_type(item) == MENU_CACHE_TYPE_SEP )
+        mi = gtk_separator_menu_item_new();
+    else
+    {
+        GtkWidget* img;
+        mi = gtk_image_menu_item_new_with_label( menu_cache_item_get_name(item) );
+        img = gtk_image_new();
+        gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM(mi), img );
+        if( menu_cache_item_get_type(item) == MENU_CACHE_TYPE_APP )
+        {
+            gtk_widget_set_tooltip_text( mi, menu_cache_item_get_comment(item) );
+            g_signal_connect( mi, "activate", on_menu_item, item );
+        }
+        g_signal_connect(mi, "map", G_CALLBACK(on_menu_item_map), item);
+        g_signal_connect(mi, "style-set", G_CALLBACK(on_menu_item_style_set), item);
+        g_signal_connect(mi, "button-press-event", G_CALLBACK(on_menu_button_press), item);
+    }
+    gtk_widget_show( mi );
+    /* g_debug("set_item_data"); */
+    g_object_set_qdata_full( G_OBJECT(mi), SYS_MENU_ITEM_ID, menu_cache_item_ref(item), menu_cache_item_unref );
+    return mi;
+}
+
+static void load_menu(MenuCacheDir* dir, GtkWidget* menu, int pos )
+{
+    GSList* l;
+    GtkWidget* mi;
+    for( l = menu_cache_dir_get_children(dir); l; l = l->next )
+    {
+        MenuCacheItem* item = MENU_CACHE_ITEM(l->data);
+        if( menu_cache_item_get_type(item) == MENU_CACHE_TYPE_APP )
+        {
+            if( is_in_lxde )
+            {
+	        if( !menu_cache_app_get_is_visible(MENU_CACHE_APP(item), SHOW_IN_LXDE) )
+                    continue;
+            }
+            else
+            {
+                /* FIXME: showing apps from all desktops is not very pleasant. */
+	        if( !menu_cache_app_get_is_visible(MENU_CACHE_APP(item), SHOW_IN_LXDE|SHOW_IN_GNOME|SHOW_IN_XFCE) )
+                    continue;
+            }
+        }
+        mi = create_item(item);
+        if( ! mi )
+            continue;
+        gtk_menu_shell_insert( (GtkMenuShell*)menu, mi, pos );
+        if( pos >= 0 )
+            ++pos;
+        if( menu_cache_item_get_type(item) == MENU_CACHE_TYPE_DIR )
+        {
+            GtkWidget* sub = gtk_menu_new();
+            load_menu( MENU_CACHE_DIR(item), sub, -1 );    /* always pass -1 for position */
+            gtk_menu_item_set_submenu( GTK_MENU_ITEM(mi), sub );
+        }
+    }
+}
+
+
+static gboolean sys_menu_item_has_data( GtkMenuItem* item )
+{
+   return (g_object_get_qdata( G_OBJECT(item), SYS_MENU_ITEM_ID ) != NULL);
+}
+
+static void unload_old_icons(GtkMenu* menu, GtkIconTheme* theme)
+{
+    GList *children, *child;
+    GtkMenuItem* item;
+    GtkWidget* sub_menu=NULL;
+
+    children = gtk_container_get_children( GTK_CONTAINER(menu) );
+    for( child = children; child; child = child->next )
+    {
+        item = GTK_MENU_ITEM( child->data );
+        if( sys_menu_item_has_data( item ) )
+        {
+            GtkImage* img;
+            item = GTK_MENU_ITEM( child->data );
+            if( GTK_IS_IMAGE_MENU_ITEM(item) )
+            {
+	        img = GTK_IMAGE(gtk_image_menu_item_get_image(GTK_IMAGE_MENU_ITEM(item)));
+                gtk_image_clear(img);
+                if( GTK_WIDGET_MAPPED(img) )
+		    on_menu_item_map(GTK_WIDGET(item),
+			(MenuCacheItem*)g_object_get_qdata(G_OBJECT(item), SYS_MENU_ITEM_ID) );
+            }
+        }
+        else if( ( sub_menu = gtk_menu_item_get_submenu( item ) ) )
+        {
+	    unload_old_icons( GTK_MENU(sub_menu), theme );
+        }
+    }
+    g_list_free( children );
+}
+
+static void remove_change_handler(gpointer id, GObject* menu)
+{
+    g_signal_handler_disconnect(gtk_icon_theme_get_default(), GPOINTER_TO_INT(id));
+}
+
+/*
+ * Insert application menus into specified menu
+ * menu: The parent menu to which the items should be inserted
+ * pisition: Position to insert items.
+             Passing -1 in this parameter means append all items
+             at the end of menu.
+ */
+static void sys_menu_insert_items( menup* m, GtkMenu* menu, int position )
+{
+    MenuCacheDir* dir;
+    guint change_handler;
+
+    if( G_UNLIKELY( SYS_MENU_ITEM_ID == 0 ) )
+        SYS_MENU_ITEM_ID = g_quark_from_static_string( "SysMenuItem" );
+
+    dir = menu_cache_get_root_dir( m->menu_cache );
+    load_menu( dir, GTK_WIDGET(menu), position );
+
+    change_handler = g_signal_connect_swapped( gtk_icon_theme_get_default(), "changed", G_CALLBACK(unload_old_icons), menu );
+    g_object_weak_ref( G_OBJECT(menu), remove_change_handler, GINT_TO_POINTER(change_handler) );
+}
+
+
 static void
-reload_system_menu( GtkMenu* menu )
+reload_system_menu( menup* m, GtkMenu* menu )
 {
     GList *children, *child;
     GtkMenuItem* item;
     GtkWidget* sub_menu;
     gint idx;
+
     children = gtk_container_get_children( GTK_CONTAINER(menu) );
-    for( child = children, idx = 0; child; child = child->next, ++idx ) {
+    for( child = children, idx = 0; child; child = child->next, ++idx )
+    {
         item = GTK_MENU_ITEM( child->data );
-        if( ptk_app_menu_item_has_data( item ) ) {
-            do {
+        if( sys_menu_item_has_data( item ) )
+        {
+            do
+            {
                 item = GTK_MENU_ITEM( child->data );
                 child = child->next;
                 gtk_widget_destroy( GTK_WIDGET(item) );
-            }while( child && ptk_app_menu_item_has_data( child->data ) );
-            ptk_app_menu_insert_items( menu, idx );
+            }while( child && sys_menu_item_has_data( child->data ) );
+            sys_menu_insert_items( m, menu, idx );
             if( ! child )
                 break;
         }
-        else if( ( sub_menu = gtk_menu_item_get_submenu( item ) ) ) {
-            reload_system_menu( GTK_MENU(sub_menu) );
+        else if( ( sub_menu = gtk_menu_item_get_submenu( item ) ) )
+        {
+            reload_system_menu( m, GTK_MENU(sub_menu) );
         }
     }
     g_list_free( children );
@@ -169,17 +554,6 @@ reload_system_menu( GtkMenu* menu )
 static void show_menu( GtkWidget* widget, Plugin* p, int btn, guint32 time )
 {
     menup* m = (menup*)p->priv;
-    /* reload system menu items if needed */
-    if( m->has_system_menu && ptk_app_menu_need_reload() ) {
-        GSList* l;
-        /* FIXME: Reload all system menus here.
-                  This is dirty, but I don't know any better way. */
-        for( l = p->panel->system_menus; l; l = l->next ) {
-            Plugin* _p = (Plugin*)l->data;
-            menup* _m = (menup*)_p->priv;
-            reload_system_menu( GTK_MENU(_m->menu) );
-        }
-    }
     gtk_menu_popup(GTK_MENU(m->menu),
                    NULL, NULL,
                    (GtkMenuPositionFunc)menu_pos, widget,
@@ -210,7 +584,7 @@ gboolean show_system_menu( gpointer system_menu )
 {
     Plugin* p = (Plugin*)system_menu;
     menup* m = (menup*)p->priv;
-    show_menu( m->bg, p, 0, GDK_CURRENT_TIME );
+    show_menu( m->img, p, 0, GDK_CURRENT_TIME );
     return FALSE;
 }
 
@@ -225,11 +599,11 @@ make_button(Plugin *p, gchar *fname, gchar *name, GdkColor* tint, GtkWidget *men
     m = (menup *)p->priv;
     m->menu = menu;
     if (p->panel->orientation == ORIENT_HORIZ) {
-        w = 10000;
         h = p->panel->ah;
+        w = h * p->panel->aw / p->panel->ah;
     } else {
         w = p->panel->aw;
-        h = 10000;
+        h = w * p->panel->ah / p->panel->aw;
     }
 
     if( name )
@@ -251,27 +625,27 @@ make_button(Plugin *p, gchar *fname, gchar *name, GdkColor* tint, GtkWidget *men
 
         /* FIXME: handle orientation problems */
         if (p->panel->usefontcolor)
-            m->bg = fb_button_new_from_file_with_colorlabel(fname, w, h, gcolor2rgb24(tint),
+            m->img = fb_button_new_from_file_with_colorlabel(fname, w, h, gcolor2rgb24(tint),
                 p->panel->fontcolor, TRUE, title);
         else
-            m->bg = fb_button_new_from_file_with_label(fname, w, h, gcolor2rgb24(tint), TRUE, title);
+            m->img = fb_button_new_from_file_with_label(fname, w, h, gcolor2rgb24(tint), TRUE, title);
 
         if( title != name )
             g_free( title );
     }
     else
     {
-        m->bg = fb_button_new_from_file(fname, w, h, gcolor2rgb24(tint), TRUE );
+        m->img = fb_button_new_from_file(fname, w, h, gcolor2rgb24(tint), TRUE );
     }
 
-    gtk_widget_show(m->bg);
-    gtk_box_pack_start(GTK_BOX(m->box), m->bg, FALSE, FALSE, 0);
+    gtk_widget_show(m->img);
+    gtk_box_pack_start(GTK_BOX(m->box), m->img, FALSE, FALSE, 0);
 
-    m->handler_id = g_signal_connect (G_OBJECT (m->bg), "button-press-event",
+    m->handler_id = g_signal_connect (G_OBJECT (m->img), "button-press-event",
           G_CALLBACK (my_button_pressed), p);
-    g_object_set_data(G_OBJECT(m->bg), "plugin", p);
+    g_object_set_data(G_OBJECT(m->img), "plugin", p);
 
-    RET(m->bg);
+    RET(m->img);
 }
 
 
@@ -363,54 +737,45 @@ read_separator(Plugin *p, char **fp)
     RET(gtk_separator_menu_item_new());
 }
 
-static gboolean on_idle( Panel* p )
+static void on_reload_menu( MenuCache* cache, menup* m )
 {
-    GSList* l;
-    /* Reload all system menus here.
-        This is dirty, but I don't know any better way. */
-    for( l = p->system_menus; l; l = l->next ) {
-        Plugin* _p = (Plugin*)l->data;
-        menup* _m = (menup*)_p->priv;
-        reload_system_menu( GTK_MENU(_m->menu) );
-    }
-    idle_loader = 0;
-    return FALSE;   /* remove the handler */
+    /* g_debug("reload system menu!!"); */
+    reload_system_menu( m, GTK_MENU(m->menu) );
 }
 
 static void
 read_system_menu(GtkMenu* menu, Plugin *p, char** fp)
 {
-   line s;
-   menup *m = (menup *)p->priv;
+    line s;
+    menup *m = (menup *)p->priv;
     GtkWidget* fake;
 
-   ENTER;
-   s.len = 256;
-   if( fp )
-   {
+    if(! m->menu_cache)
+    {
+        if( !g_getenv("XDG_MENU_PREFIX") )
+            g_setenv("XDG_MENU_PREFIX", "lxde-", TRUE);
+        m->menu_cache = menu_cache_lookup( "applications.menu" );
+        if( G_UNLIKELY(!m->menu_cache) )
+        {
+            ERR("error loading applications menu");
+            return;
+        }
+        m->reload_notify = menu_cache_add_reload_notify(m->menu_cache, on_reload_menu, m);
+    }
+
+    s.len = 256;
+    if( fp )
+    {
         while (lxpanel_get_line(fp, &s) != LINE_BLOCK_END) {
             ERR("menu: error - system can not have paramteres\n");
-            RET();
+            return;
         }
-   }
+    }
 
-   /* ptk_app_menu_insert_items( menu, -1 ); */
-   /* Don't load the real system menu here to speed up startup.
-    * Let's add a fake item to cheat PtkAppMenu as a place holder,
-    * and we utilize reload_system_menu() to load the real menu later. */
-    fake = gtk_separator_menu_item_new();
-    PTK_APP_MENU_ITEM_ID = g_quark_from_static_string( "PtkAppMenuItem" );
-    g_object_set_qdata( fake, PTK_APP_MENU_ITEM_ID, GUINT_TO_POINTER(TRUE) );
-   gtk_menu_shell_append( menu, fake);
+    sys_menu_insert_items( m, menu, -1 );
+    m->has_system_menu = TRUE;
 
-   m->has_system_menu = TRUE;
-
-   p->panel->system_menus = g_slist_append( p->panel->system_menus, p );
-
-    if( idle_loader == 0 )  /* delay the loading, and do it in idle handler */
-        idle_loader = g_idle_add( (GSourceFunc)on_idle, p->panel );
-
-   RET();
+    p->panel->system_menus = g_slist_append( p->panel->system_menus, p );
 }
 
 static void
@@ -461,7 +826,6 @@ read_submenu(Plugin *p, char** fp, gboolean as_item)
 
     ENTER;
 
-
     s.len = 256;
     menu = gtk_menu_new ();
     gtk_container_set_border_width(GTK_CONTAINER(menu), 0);
@@ -499,7 +863,7 @@ read_submenu(Plugin *p, char** fp, gboolean as_item)
                 fname = expand_tilda(s.t[1]);
             else if (!g_ascii_strcasecmp(s.t[0], "name"))
                 strcpy(name, s.t[1]);
-	    /* FIXME: tintcolor will not be saved.  */
+        /* FIXME: tintcolor will not be saved.  */
             else if (!g_ascii_strcasecmp(s.t[0], "tintcolor"))
                 gdk_color_parse( s.t[1], &color);
             else {
@@ -531,7 +895,7 @@ read_submenu(Plugin *p, char** fp, gboolean as_item)
         gtk_menu_item_set_submenu (GTK_MENU_ITEM (mi), menu);
         RET(mi);
     } else {
-        m->fname = g_strdup(fname);
+        m->fname = fname ? g_strdup(fname) : g_strdup( PACKAGE_DATA_DIR "/lxpanel/images/my-computer.png" );
         m->caption = g_strdup(name);
         mi = make_button(p, fname, name, &color, menu);
         if (fname)
@@ -550,9 +914,9 @@ read_submenu(Plugin *p, char** fp, gboolean as_item)
 static int
 menu_constructor(Plugin *p, char **fp)
 {
+    char *start;
     menup *m;
     static char default_config[] =
-        "image=" PACKAGE_DATA_DIR "/lxpanel/images/my-computer.png\n"
         "system {\n"
         "}\n"
         "separator {\n"
@@ -568,15 +932,16 @@ menu_constructor(Plugin *p, char **fp)
         "}\n"
         "}\n";
     char *config_start, *config_end, *config_default = default_config;
+    int iw, ih;
 
-    ENTER;
     m = g_new0(menup, 1);
     g_return_val_if_fail(m != NULL, 0);
     m->fname = NULL;
     m->caption = NULL;
+
     p->priv = m;
 
-    //gtk_rc_parse_string(menu_rc);
+/*
     if  (p->panel->orientation == ORIENT_HORIZ)
         m->paneliconsize = p->panel->ah
             - 2* GTK_WIDGET(p->panel->box)->style->ythickness;
@@ -584,6 +949,9 @@ menu_constructor(Plugin *p, char **fp)
         m->paneliconsize = p->panel->aw
             - 2* GTK_WIDGET(p->panel->box)->style->xthickness;
     m->iconsize = 22;
+*/
+    gtk_icon_size_lookup( GTK_ICON_SIZE_MENU, &iw, &ih );
+    m->iconsize = MAX(iw, ih);
 
     m->box = gtk_hbox_new(FALSE, 0);
     gtk_container_set_border_width(GTK_CONTAINER(m->box), 0);
@@ -591,7 +959,7 @@ menu_constructor(Plugin *p, char **fp)
     if( ! fp )
         fp = &config_default;
 
-    m->config_start = *fp;
+    m->config_start = start = *fp;
     if (!read_submenu(p, fp, FALSE)) {
         ERR("menu: plugin init failed\n");
         goto error;
@@ -603,8 +971,7 @@ menu_constructor(Plugin *p, char **fp)
     if( *m->config_end == '}' )
         --m->config_end;
 
-    m->config_data = g_strndup( m->config_start,
-                                (m->config_end-m->config_start) );
+    m->config_data = g_strndup( start, (m->config_end - start) );
 
     p->pwid = m->box;
 
@@ -618,6 +985,7 @@ menu_constructor(Plugin *p, char **fp)
 static void save_config( Plugin* p, FILE* fp )
 {
     menup* menu = (menup*)p->priv;
+    int level = 0;
     lxpanel_put_str( fp, "name", menu->caption );
     lxpanel_put_str( fp, "image", menu->fname );
     if( menu->config_data ) {
@@ -626,7 +994,20 @@ static void save_config( Plugin* p, FILE* fp )
         for( line = lines; *line; ++line ) {
             g_strstrip( *line );
             if( **line )
+            {
+                if( level == 0 )
+                {
+                    /* skip image and caption since we already save these two items */
+                    if( g_str_has_prefix(*line, "image") || g_str_has_prefix(*line, "caption") )
+                        continue;
+                }
+                g_strchomp(*line); /* remove trailing spaces */
+                if( g_str_has_suffix( *line, "{" ) )
+                    ++level;
+                else if( g_str_has_suffix( *line, "}" ) )
+                    --level;
                 lxpanel_put_line( fp, *line );
+            }
         }
         g_strfreev( lines );
     }
@@ -634,7 +1015,9 @@ static void save_config( Plugin* p, FILE* fp )
 
 static void apply_config(Plugin* p)
 {
-    /* FIXME: update menu for new setting */
+    menup* m = (menup*)p->priv;
+    if( m->fname )
+        fb_button_set_from_file( m->img, m->fname );
 }
 
 static void menu_config( Plugin *p, GtkWindow* parent )
@@ -644,8 +1027,8 @@ static void menu_config( Plugin *p, GtkWindow* parent )
     dlg = create_generic_config_dlg( _(p->class->name),
                                      GTK_WIDGET(parent),
                                     (GSourceFunc) apply_config, (gpointer) p,
-                                     _("Icon"), &menu->fname, G_TYPE_STRING,
-                                     _("Caption"), &menu->caption, G_TYPE_STRING,
+                                     _("Icon"), &menu->fname, CONF_TYPE_FILE_ENTRY,
+                                     /* _("Caption"), &menu->caption, CONF_TYPE_STR, */
                                      NULL );
     gtk_window_present( GTK_WINDOW(dlg) );
 }
@@ -656,12 +1039,12 @@ PluginClass menu_plugin_class = {
 
     type : "menu",
     name : N_("Menu"),
-    version: "1.0",
+    version: "2.0",
     description : N_("Provide Menu"),
 
     constructor : menu_constructor,
     destructor  : menu_destructor,
-    /* config : menu_config, */
+    config : menu_config,
     save : save_config
 };
 
